#include "osHandles.h"              // Includes all OS Files
#include "System/cpu.h"             // Initialize CPU Hardware
#include "drivers/watchdog.h"
#include "drivers/uart0.h"       	// Initialize UART
#include "utils/rprintf.h"			// Reduced printf.  STDIO uses lot of FLASH space & Stack space
#include "fat/diskio.h"
#include "drivers/i2c.h"
#include "drivers/pcm1774.h"
#include "drivers/sta013.h"
#include "fat/ff.h"
#include "utils/cString.h"
#include "utils/vector.hpp"
#include "drivers/pwm.h"
#include "drivers/motor.h"
#include "adjd_S311/adjd_s311.hpp"
#include "adjd_S311/carControl.h"

#include "general/userInterface.hpp"	// User interface functions to interact through UART
#include "utils/timer.hpp"

void FatFsDiskTimer(void* p)
{
	disk_timerproc();
}

/* INTERRUPT VECTORS:
 * 0:    OS Timer Tick
 * 1:    Not Used
 * 2:    UART0 Interrupt
 * 3:    Not Used
 * 4:    I2C0 Interrupt
 * 5-16: Not Used
 */
int main(void)
{
	OSHANDLES SysHandles; // Should contain all OS Handles

	cpuSetupHardware(); // Setup PLL, enable MAM etc.
	watchdogDelayUs(1000 * 1000); // Some startup delay
	uart0Init(38400, 128); // 128 is size of Rx/Tx FIFO
	initialize_PWM();
	initialize_motor();

	// Use polling version of uart0 to do printf/rprintf before starting FreeRTOS
	rprintf_devopen(uart0PutCharPolling);
	cpuPrintMemoryInfo();

	// Open interrupt-driven version of UART0 Rx/Tx
	rprintf_devopen(uart0PutChar);
	watchdogDelayUs(1000 * 1000);

	/** Create timer needed for SD Card I/O */
	Timer diskTimer(FatFsDiskTimer, 10, TimerPeriodic);
	diskTimer.start();

	/** Create any Queues and Mutexes **/
	SysHandles.lock.SPI = xSemaphoreCreateMutex();
	SysHandles.queue.stop = xQueueCreate(1,5);
	SysHandles.queue.colorValue = xQueueCreate(1,4);
	SysHandles.queue.initValues = xQueueCreate(9,4);


	if (	pdPASS
			!= xTaskCreate( uartUI, (signed char*)"Uart UI", STACK_BYTES(1024*10), &SysHandles, PRIORITY_LOW, &SysHandles.task.userInterface )
			|| pdPASS
			!= xTaskCreate( getColor,(signed char*)"getColor", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.getColor)
			|| pdPASS
			!= xTaskCreate( controlCar,(signed char*)"control", STACK_BYTES(1024*6), &SysHandles, PRIORITY_LOW, &SysHandles.task.controlCar)
)
	{
		rprintf(
				"ERROR:  OUT OF MEMORY: Check OS Stack Size or task stack size.\n");
	}

	// Start FreeRTOS to begin servicing tasks created above, vTaskStartScheduler() will not "return"
	rprintf("\n-- Starting FreeRTOS --\n");
	vTaskStartScheduler();

	// In case OS is exited:
	rprintf_devopen(uart0PutCharPolling);
	rprintf("ERROR: Unexpected OS Exit!\n");

	return 0;
}
